# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2018, 2020 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------
#

import os
import sys
import re
import subprocess
import logging
from config_system import get_config_bool, get_config_string, set_config, get_config, get_config_list
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "../bob/scripts"))
import kernel_config_parser

logger = logging.getLogger(__name__)

g_kernel_version = None
def kernel_get_version(kdir):
    global g_kernel_version

    if g_kernel_version:
        return g_kernel_version

    cmd = ['make', '-s', 'kernelversion']
    try:
        version = subprocess.check_output(cmd, cwd = kdir).strip()
        version = version.decode(sys.getdefaultencoding())
    except (OSError, subprocess.CalledProcessError) as e:
        logger.warning("Couldn't get kernel version: %s" % str(e))
        version = "0.0.0"

    m = re.match(r'([0-9]+)(?:\.([0-9]+)(?:\.([0-9]+))?)?(.*)', version)
    if not m:
        logger.warning("Couldn't parse kernel version '%s'" % version)

    version_ints = []
    for i in m.groups()[0:3]:
        if i != None and i.isdigit():
            version_ints.append(int(i))
        else: # Not set
            version_ints.append(0)

    g_kernel_version = tuple(version_ints)
    return g_kernel_version

def disable_devfreq_if_not_supported(kdir):
    if not get_config_bool('MALI_DEVFREQ'):
        return

    if kernel_get_version(kdir) >= (3, 18, 0):
        if kernel_config_parser.option_enabled(kdir, 'CONFIG_PM_DEVFREQ'):
            # Leave MALI_DEVFREQ at its current value
            return
        else:
            logger.warning("Disabling MALI_DEVFREQ - CONFIG_PM_DEVFREQ not enabled in kernel")
    else:
        logger.warning("Disabling MALI_DEVFREQ - kernel is too old")
    set_config('MALI_DEVFREQ', False)

def bsp_xen_supported(kdir):
    config_list = get_config_list()
    if 'BSP_HAS_HYPERVISOR' in config_list:
        bsp = get_config('BSP_HAS_HYPERVISOR')
        if bsp['is_user_set']:
            return

        if kernel_config_parser.option_enabled(kdir, 'CONFIG_XEN'):
            logger.debug("Enabling BSP_HAS_HYPERVISOR - Xen detected in kernel")
            set_config('BSP_HAS_HYPERVISOR', True, is_user_set=False)
        else:
            logger.debug("Disabling BSP_HAS_HYPERVISOR - CONFIG_XEN not enabled in kernel")
            set_config('BSP_HAS_HYPERVISOR', False, is_user_set=False)

def dma_buf_sync_supported(kdir):
    if kdir != "" and kernel_get_version(kdir) >= (4, 6, 0):
        set_config('DMA_BUF_SYNC_IOCTL_SUPPORTED', True)
    else:
        set_config('DMA_BUF_SYNC_IOCTL_SUPPORTED', False)

def plugin_exec():
    if not get_config_bool('ALLOW_KERNEL_EXPLORE'):
        return

    kdir = get_config_string('KERNEL_DIR')
    if get_config_bool('BUILD_KERNEL_MODULES') or get_config_bool('USERSPACE_INCLUDES_KERNEL_HEADERS'):
        if kdir == "":
            logger.warning("KERNEL_DIR not set, kernel exploration disabled")
            return
    if get_config_bool('BUILD_KERNEL_MODULES'):
        disable_devfreq_if_not_supported(kdir)
        bsp_xen_supported(kdir)
    dma_buf_sync_supported(kdir)
